

#include "header.fx"

float4x4 g_WorldViewProj : WORLDVIEWPROJ;

float3 g_SunLightColor;
float3 g_TorchLightColor;

struct VS_INPUT
{
    float3 pos        : POSITION;
    float3 normal        : NORMAL;
    float4 color         : COLOR;
    float2 uv0            : TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 pos            : POSITION;
    float4 color0        : COLOR0; //color0.xyz=ambient lighting,  color0.w=specular lighting
    float2 uv0            : TEXCOORD0;

#if COLORMAP>0
    float2 cuv : TEXCOORD1;  //color map
#endif

#ifdef LIGHT_SHADOW
    float4 color1        : COLOR1; //color1.xyz=shadowed_lighting,  color1.w=specular lighting
    float4 uv1            : TEXCOORD2;
#endif

#if FOG_HEIGHT>0 || FOG_DISTANCE>0
	float2 fogc	: TEXCOORD3;
#endif
};

VS_OUTPUT VSMain(VS_INPUT input)
{
	VS_OUTPUT output;
    
	float3 pos        = input.pos.xyz;
	float3 normal    = input.normal * float3(2/255.0, 2/255.0, 2/255.0) - float3(1.0, 1.0, 1.0);
	float2 uv0 = input.uv0 * float2(1.0/255.0, 1.0/255.0);
	float4 color = input.color * float4(1.0/255.0, 1.0/255.0, 1.0/255.0, 1.0/255.0);
	float3 ambient = max(color.x*g_SunLightColor, color.y*g_TorchLightColor);

	output.pos = mul(float4(pos,1.0), g_WorldViewProj);
	output.color0 = float4(ambient, 1.0f);  //DoLighting(pos, normal, 0);
	output.uv0 = uv0;

#if COLORMAP>0
	output.cuv = float2(color.zw);
#endif

#ifdef LIGHT_SHADOW
	output.color1 = DoLighting(pos, normal, 1);
   
	float4 wpos2 = mul(float4(pos,1.0), g_World);
	output.uv1 = CalShadowProjUV(wpos2);
#endif


#if FOG_HEIGHT>0 || FOG_DISTANCE>0
	float3 wpos = mul(float4(pos,1.0), g_World);
	output.fogc = DoFog(wpos);
#endif

	return output;
}
